-----------------------
-- USB test version 2
-----------------------

-- Description -----------------------------------
-- USB echo test
--  READstart -(VALID)-> READdone -(BUSY)-> 
--  WRITEstart -(VALID)-> WRITEdone -(BUSY)-> back
-------------------------------------------------

---- Notes --------------------------------------------------------
-- 2006.12.04
--   SɏB
--   z50MHz̃NbNB
--
-------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity USB_TEST is
    port(
        CLKIN : in std_logic;
        LED : out std_logic_vector(7 downto 0);

        -- to USB
        USB_RSTX : out std_logic;                        -- USB reset
        USB_RDX : out std_logic;                         -- 0 when read
        USB_WR : out std_logic;                          -- 1 when write
        USB_TXEX : in std_logic;                         -- 1 when buffer is full
        USB_RXFX : in std_logic;                         -- 1 when buffer is empty
        USB_DATA : inout std_logic_vector(7 downto 0));   -- USB data

end USB_TEST;

architecture RTL of USB_TEST is

    component ROC
        port(
            O : out std_logic);
    end component;

    component IBUFG
        port(
            I: in std_logic;
            O: out std_logic);
    end component;

    component USB
        port(
        CLK : in std_logic;                          -- clock 50MHz
        RESET : in std_logic;                        -- reset signal
        RMODE : in std_logic;                        -- 1 when read
        WMODE : in std_logic;                        -- 1 when write
        RDATA : out std_logic_vector(7 downto 0);    -- read data
        WDATA : in std_logic_vector(7 downto 0);     -- write data
        VALID : out std_logic;                       -- 1 when read is O.K.
        BUSY : out std_logic;
        -- to USB
        RSTX : out std_logic;                        -- USB reset
        RDX : out std_logic;                         -- 0 when read
        WR : out std_logic;                          -- 1 when write
        TXEX : in std_logic;                         -- 1 when buffer is full
        RXFX : in std_logic;                         -- 1 when buffer is empty
        USBDATA : inout std_logic_vector(7 downto 0));  -- USB data
    end component;

    signal CLK : std_logic;
    signal RESET : std_logic;

    signal RMODE : std_logic;
    signal WMODE : std_logic;
    signal RDATA : std_logic_vector(7 downto 0);
    signal WDATA : std_logic_vector(7 downto 0);
    signal VALID : std_logic;
    signal BUSY : std_logic;

    signal STATE : std_logic_vector(1 downto 0);

begin
    ROC : ROC port map (
        O => RESET);

    -- ??
    IBUFG : IBUFG port map (
        I => CLKIN,
        O => CLK);
  
    USB : USB port map (
        CLK      => CLK,
        RESET    => RESET,
        RMODE    => RMODE,
        WMODE    => WMODE,
        RDATA    => RDATA,
        WDATA    => WDATA,
        VALID    => VALID,
        BUSY     => BUSY,
        RSTX   => USB_RSTX,
        RDX      => USB_RDX,
        WR       => USB_WR,
        TXEX     => USB_TXEX,
        RXFX     => USB_RXFX,
        USBDATA => USB_DATA);

    process(CLK,RESET)
    begin
        if RESET = 1 then
            RMODE <= '0';
            WMODE <= '0';
            WDATA <= "ZZZZZZZZ";
            LED <= "11111111";
        elsif CLK'event and CLK = '1' then
            case STATE is
                when "00" =>
                    LED <= "11111110";
                    if BUSY = '0' then
                        RMODE <= '1';
                        WMODE <= '0';
                        STATE <= STATE + '1';
                    else
                        STATE <= "00";
                    end if;
                when "01" =>
                    LED <= "11111101";
                    if VALID = '1' then
                        RMODE <= '0';
                        WDATA <= RDATA;
                        STATE <= STATE + '1';
                    else
                        STATE <= "01";
                    end if;
                when "10" =>
                    LED <= "11111100";
                    if BUSY = '0' then
                        WMODE <= '1';
                        STATE <= STATE + '1';
                    else
                        STATE <= "10";
                    end if;
                when "11" =>
                    LED <= "11111011";
                    if VALID = '1' then
                        WMODE <= '0';
                        STATE <= STATE + '1';
                    else
                        STATE <= "11";
                    end if;
                when others =>
                    LED <= "00000000";
                    RMODE <= '0';
                    WMODE <= '0';
                    WDATA <= "ZZZZZZZZ";
                    STATE <= "00";
                end case;
        end if;  -- CLK
    end process;
end USB_TEST;
