library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity USB_TEST is
  
  port (
    CLK        : in    std_logic;                     -- clock
    LED        : out   std_logic_vector(7 downto 0);  -- debug
    -- usb
    USB_NRESET : out   std_logic;                     -- reset
    USB_NRD    : out   std_logic;
    USB_WR     : out   std_logic;
    USB_NTXE   : in    std_logic;
    USB_NRXF   : in    std_logic;
    USB_DATA   : inout std_logic_vector(7 downto 0));

end USB_TEST;

architecture USB_TEST_arc of USB_TEST is

  component ROC
    port (
      O : out std_logic);
  end component;

  component IBUFG
    port (
      I : in  std_logic;
      O : out std_logic);
  end component;
  
  component USB
    port (
      CLK      : in    std_logic;
      RESET    : in    std_logic;
      DATA_IN  : in    std_logic_vector(7 downto 0);
      DATA_OUT : out   std_logic_vector(7 downto 0);
      USBIN    : in    std_logic;
      USBOUT   : in    std_logic;
      BUSY     : out   std_logic;
      VALID    : out   std_logic;
      NRESET   : out   std_logic;
      NRD      : out   std_logic;
      WR       : out   std_logic;      
      NTXE     : in    std_logic;
      NRXF     : in    std_logic;
      USB_DATA : inout std_logic_vector(7 downto 0));
  end component;

  signal DATA : std_logic_vector(7 downto 0);
  signal USB_IN,USB_OUT : std_logic;
  signal BUSY : std_logic;
  signal VALID : std_logic;
  signal STATE : std_logic_vector(3 downto 0);
  signal CLKIN : std_logic;
  signal RESET : std_logic;
  
begin  -- USB_TEST_arc

  ROC : ROC port map (
    O => RESET);

  IBUFG : IBUFG port map (
    I => CLK,
    O => CLKIN);
  
  USB : USB port map (
    CLK      => CLK,
    RESET    => RESET,
    DATA_IN  => DATA,
    DATA_OUT => DATA,
    USBIN   => USB_IN,
    USBOUT  => USB_OUT,
    BUSY     => BUSY,
    VALID    => VALID,
    NRESET   => USB_NRESET,
    NRD      => USB_NRD,
    WR       => USB_WR,
    NTXE     => USB_NTXE,
    NRXF     => USB_NRXF,
    USB_DATA => USB_DATA);

  usb_test: process (CLK, RESET)
  begin  -- process usb_test
    if RESET = '1' then                 -- asynchronous reset (active low)
      USB_IN <= '0';
      USB_OUT <= '0';
      STATE <= "0000";
      DATA <= "ZZZZZZZZ";
      LED <= "0000000";                 -- light on
    elsif CLK'event and CLK = '1' then  -- rising clock edge
      if BUSY = '0' then
        USB_IN <= '0';
        USB_OUT <= '1';
        LED <= "11111110";
      elsif VALID = '1' then
        USB_OUT <= '0';
        USB_IN <= '1';
        LED <= "11111101";
      else
        USB_IN <= '0';
        USB_OUT <= '0';
        LED <= "11111011";
      end if;
    end if;
  end process usb_test;  

end USB_TEST_arc;

















